@@ -987,7 +987,7 @@ class DelayedArgument(LazyEvaluation):
         raise ArgumentTypeError(F'The meta variable {name} is of type {type(obj).__name__} and no conversion is known.')
 
     def _var(self, name: str, eat: bool) -> bytes:
-        if not name.isidentifier():
+        if not is_valid_variable_name(name, allow_wildcards=True):
             name = multibin(name)
 
         def extract(data: Chunk):
@@ -997,6 +997,11 @@ class DelayedArgument(LazyEvaluation):
                     var = bytes(var)
                 var = var.decode()
             meta = metavars(data)
+            if var not in meta:
+                from fnmatch import fnmatch as matches
+                options = [name for name in meta.keys() if matches(name, var)]
+                if len(options) == 1:
+                    var = options[0]
             try:
                 if eat:
                     result = meta.pop(var)
@@ -1012,8 +1017,9 @@ class DelayedArgument(LazyEvaluation):
     def var(self, name: str) -> bytes:
         """
         The handler `var:name` contains the value of the meta variable `name`. This handler is semi-final;
-        if the provided argument is an identifier, it is read directly as a variable name. If it is not an
-        identifier, it will be interpreted as a multibin expression to compute the name.
+        if the provided argument is an identifier pattern, it is read directly as a variable name. Otherwise,
+        it will be interpreted as a multibin expression to compute the name. An identifier pattern here is
+        a unix file name pattern including the wildcards `?`, `*`, and letter options in `[` brackets `]`.
         """
         return self._var(name, eat=False)
 
@@ -1383,6 +1389,20 @@ class DelayedBinaryArgument(DelayedArgument):
         return value
 
 
+class DelayedPathArgument(DelayedBinaryArgument):
+    """
+    A parser for binary arguments like `refinery.lib.argformats.DelayedBinaryArgument` which does not
+    read files from disk as the default. This makes it more suitable for parsing path patterns that
+    are passed to path extractor units: It avoids the problem that a provided file name is confused
+    with the contents of an equally named file on disk.
+    """
+    def default_handler(self, expression: str) -> bytes:
+        try:
+            return utf8(expression)
+        except Exception:
+            return expression
+
+
 class DelayedNumSeqArgument(DelayedArgument):
     """
     A parser for sequences of numeric arguments. It does not implement any handlers beyond the default
@@ -1478,11 +1498,14 @@ class DelayedRegexpArgument(DelayedArgument):
         """
         if '(?' in expression:
             from refinery.lib.patterns import formats, indicators
+            tick = 0
 
-            def replace_known_pattern(match):
-                name = match[1]
-                return '(?:{})'.format(formats.get(
-                    name, indicators.get(name, match[0])))
+            def replace_known_pattern(match: re.Match[str]):
+                nonlocal tick
+                tick += 1
+                pattern = str(formats.get(match[1], indicators.get(match[1], match[0])))
+                pattern = re.sub(r'(?<=\(\?P[<=])__(\w+)__', F'__\\1_{tick}__', pattern)
+                return '(?:{})'.format(pattern)
 
             def replace_variable_assignment(match):
                 return F'(?P<{match[1]}>'
@@ -1491,8 +1514,8 @@ class DelayedRegexpArgument(DelayedArgument):
                 replace_variable_assignment, expression)
             expression = re.sub(
                 R'\(\?\?({}|{})\)'.format(
-                    '|'.join(p.name for p in formats),
-                    '|'.join(p.name for p in indicators)
+                    '|'.join(re.escape(p.name) for p in formats),
+                    '|'.join(re.escape(p.name) for p in indicators)
                 ),
                 replace_known_pattern,
                 expression
@@ -1594,7 +1617,7 @@ class DelayedNumberArgument(DelayedArgument):
         return LazyPythonExpression(expression)
 
 
-class number:
+class ___number:
     __name__ = 'number'
 
     def __init__(self, min=None, max=None):
@@ -1621,7 +1644,7 @@ class number:
             return number(F'0x{match[1]}')
 
 
-number = number()
+number = ___number()
 """
 The singleton instance of a class that uses `refinery.lib.argformats.PythonExpression`
 to parse expressions with integer value. This singleton can be slice accessed to
@@ -1642,6 +1665,18 @@ def numseq(expression: Union[int, str], reverse=False, seed=None, typecheck=True
     return arg
 
 
+def pathvar(expression: Union[str, bytes, bytearray]) -> Union[bytes, DelayedArgument]:
+    """
+    This is the argument parser type that uses `refinery.lib.argformats.DelayedPathArgument`.
+    """
+    if not isinstance(expression, str):
+        return bytes(expression)
+    arg = DelayedPathArgument(expression)
+    with suppress(TooLazy):
+        return arg()
+    return arg
+
+
 def multibin(expression: Union[str, bytes, bytearray], reverse=False, seed=None) -> Union[bytes, DelayedArgument]:
     """
     This is the argument parser type that uses `refinery.lib.argformats.DelayedBinaryArgument`.
