@@ -9,17 +9,18 @@ from typing import TYPE_CHECKING
 
 from abc import ABC, abstractmethod
 from enum import IntFlag
-from functools import lru_cache, cached_property, partial
+from functools import cached_property, partial
 
 from refinery.lib.executable import align, Arch, ET, BO, Executable, ExecutableCodeBlob
 from refinery.lib.tools import NoLogging
 from refinery.lib.vfs import VirtualFileSystem
 from refinery.lib.intervals import IntIntervalUnion
-from refinery.units import RefineryImportMissing
+from refinery.lib.exceptions import MissingModule
 
 if TYPE_CHECKING:
     from capstone import Cs
     from speakeasy import Speakeasy as Se
+    from speakeasy.common import CodeHook, Hook as SeHook
     from speakeasy.memmgr import MemMap
     from unicorn import Uc
     from icicle import Icicle as Ic
@@ -35,21 +36,6 @@ _E = TypeVar('_E')
 _R = TypeVar('_R')
 
 
-class MissingModule:
-    """
-    This class can wrap a module import that is currently missing. If any attribute of the missing
-    module is accessed, it raises `refinery.units.RefineryImportMissing`.
-    """
-    def __init__(self, name, dist=None):
-        self.name = name
-        self.dist = dist or name
-
-    def __getattr__(self, key: str):
-        if key.startswith('__') and key.endswith('__'):
-            raise AttributeError(key)
-        raise RefineryImportMissing(self.name, self.dist)
-
-
 try:
     with NoLogging():
         import unicorn as uc
@@ -151,7 +137,7 @@ class Emulator(ABC, Generic[_E, _R, _T]):
         if isinstance(data, Executable):
             exe = data
         try:
-            exe = Executable.Load(data)
+            exe = Executable.Load(data, base)
         except ValueError:
             exe = ExecutableCodeBlob(data, base, arch)
             raw = True
@@ -213,6 +199,40 @@ class Emulator(ABC, Generic[_E, _R, _T]):
         self.state = state
         self._reset()
 
+    def step(self, address: int, count: int = 1) -> int:
+        """
+        This method emulates `count` many instructions starting at `address`. Returns the current
+        instruction pointer value after stepping.
+        """
+        if not self._resetonce:
+            self.reset()
+        try:
+            self._enable_single_step()
+            for _ in range(count):
+                self.emulate(address)
+                address = self.ip
+            return address
+        finally:
+            self._disable_single_step()
+
+    def base_exe_to_emu(self, address: Optional[int]):
+        """
+        Rebase a virtual address from the base executable's address space to the one used by the
+        emulator.
+        """
+        if address is not None:
+            address = address - self.exe.base + self.base
+        return address
+
+    def base_emu_to_exe(self, address: Optional[int]):
+        """
+        Rebase a virtual address from the emulator's address space to the one used by the base
+        executable.
+        """
+        if address is not None:
+            address = address + self.exe.base - self.base
+        return address
+
     def emulate(self, start: int, end: Optional[int] = None):
         """
         Call this function to begin emulation. The `start` parameter is the address where execution
@@ -220,10 +240,6 @@ class Emulator(ABC, Generic[_E, _R, _T]):
         """
         if not self._resetonce:
             self.reset()
-        exe = self.exe
-        start = start - exe.base + self.base
-        if end is not None:
-            end = end - exe.base + self.base
         return self._emulate(start, end)
 
     def call(self, function: int):
@@ -316,6 +332,20 @@ class Emulator(ABC, Generic[_E, _R, _T]):
         """
         ...
 
+    @abstractmethod
+    def _enable_single_step(self):
+        """
+        Enable single stepping.
+        """
+        ...
+
+    @abstractmethod
+    def _disable_single_step(self):
+        """
+        Enable single stepping.
+        """
+        ...
+
     @abstractmethod
     def morestack(self):
         """
@@ -526,37 +556,12 @@ class Emulator(ABC, Generic[_E, _R, _T]):
         """
         Disassemble a single instruction at the given address.
         """
-        ea = address - self.base + self.exe.base
-        cs = self.disassembler()
+        if not self._resetonce:
+            self.reset()
+        cs = self.exe.disassembler()
         cs.detail = True
-        pa = self.exe.location_from_address(ea).physical.position
-        return next(cs.disasm(bytes(self.exe.data[pa:pa + 0x20]), address, 1))
-
-    @lru_cache
-    def disassembler(self) -> Cs:
-        """
-        Create a capstone disassembler that matches the emulator's architecture.
-        """
-        cs_arch, cs_mode = {
-            Arch.X32     : (cs.CS_ARCH_X86,   cs.CS_MODE_32),     # noqa
-            Arch.X64     : (cs.CS_ARCH_X86,   cs.CS_MODE_64),     # noqa
-            Arch.ARM32   : (cs.CS_ARCH_ARM,   cs.CS_MODE_ARM),    # noqa
-            Arch.ARM64   : (cs.CS_ARCH_ARM,   cs.CS_MODE_THUMB),  # noqa
-            Arch.MIPS16  : (cs.CS_ARCH_MIPS,  cs.CS_MODE_16),     # noqa
-            Arch.MIPS32  : (cs.CS_ARCH_MIPS,  cs.CS_MODE_32),     # noqa
-            Arch.MIPS64  : (cs.CS_ARCH_MIPS,  cs.CS_MODE_64),     # noqa
-            Arch.PPC32   : (cs.CS_ARCH_PPC,   cs.CS_MODE_32),     # noqa
-            Arch.PPC64   : (cs.CS_ARCH_PPC,   cs.CS_MODE_64),     # noqa
-            Arch.SPARC32 : (cs.CS_ARCH_SPARC, cs.CS_MODE_32),     # noqa
-            Arch.SPARC64 : (cs.CS_ARCH_SPARC, cs.CS_MODE_V9),     # noqa
-        }[self.exe.arch()]
-
-        cs_mode |= {
-            BO.BE: cs.CS_MODE_BIG_ENDIAN,
-            BO.LE: cs.CS_MODE_LITTLE_ENDIAN,
-        }[self.exe.byte_order()]
-
-        return cs.Cs(cs_arch, cs_mode)
+        data = self.mem_read(address, 0x20)
+        return next(cs.disasm(data, address, 1))
 
     def general_purpose_registers(self):
         """
@@ -619,6 +624,7 @@ class RawMetalEmulator(Emulator[_E, _R, _T]):
                 continue
             base = self.align(segment.virtual.lower, down=True)
             size = self.align(segment.virtual.upper) - base
+            size = max(size, len(segment.physical))
             mem.addi(base, size)
         it = iter(mem)
         for interval in it:
@@ -671,6 +677,7 @@ class UnicornEmulator(RawMetalEmulator[Uc, int, _T]):
         }[self.exe.byte_order()]
 
         self.unicorn = uc.Uc(uc_arch, uc_mode)
+        self._single_step_hook = None
 
         self._map_segments()
         self._map_stack_and_heap()
@@ -688,6 +695,27 @@ class UnicornEmulator(RawMetalEmulator[Uc, int, _T]):
             if self.hooked(flag):
                 self.unicorn.hook_add(hook, callback, user_data=self.state)
 
+    class _singlestep:
+        def __init__(self):
+            self.stepped = False
+
+        def __call__(self, uc: Uc, *_, **kw):
+            if self.stepped:
+                self.stepped = False
+                uc.emu_stop()
+            else:
+                self.stepped = True
+
+    def _enable_single_step(self):
+        if self._single_step_hook is not None:
+            return
+        self._single_step_hook = self.unicorn.hook_add(uc.UC_HOOK_CODE, self._singlestep())
+
+    def _disable_single_step(self):
+        if hook := self._single_step_hook:
+            self.unicorn.hook_del(hook)
+            self._single_step_hook = None
+
     def _init(self):
         self._reg_by_name: Dict[str, Register[int]] = {}
         self._reg_by_code: Dict[int, Register[int]] = {}
@@ -762,7 +790,7 @@ class IcicleEmulator(RawMetalEmulator[Ic, str, _T]):
     icicle: Ic
 
     def _init(self):
-        ...
+        self._single_step = False
 
     def _reset(self):
         exe = self.exe
@@ -789,6 +817,12 @@ class IcicleEmulator(RawMetalEmulator[Ic, str, _T]):
         self._map_segments()
         self._map_stack_and_heap()
 
+    def _enable_single_step(self):
+        self._single_step = True
+
+    def _disable_single_step(self):
+        self._single_step = False
+
     def _emulate(self, start: int, end: Optional[int] = None):
         dasm = self.disassembler()
         code = False
@@ -845,11 +879,11 @@ class IcicleEmulator(RawMetalEmulator[Ic, str, _T]):
             perm = MP.ExecuteReadWrite
         return self.icicle.mem_map(address, size, perm)
 
-    def _set_register(self, reg: str, value: int) -> None:
-        return self.icicle.reg_write(reg, value)
+    def _set_register(self, register: str, v: int) -> None:
+        return self.icicle.reg_write(register, v)
 
-    def _get_register(self, reg: str) -> int:
-        return self.icicle.reg_read(reg)
+    def _get_register(self, register: str) -> int:
+        return self.icicle.reg_read(register)
 
     def mem_write(self, address: int, data: bytes):
         return self.icicle.mem_write(address, data)
@@ -869,6 +903,32 @@ class SpeakeasyEmulator(Emulator[Se, str, _T]):
     def _init(self):
         self._regs: Dict[str, Register[str]] = {}
 
+    class _singlestep:
+        def __init__(self):
+            self.stepped = False
+
+        def __call__(self, se: Se, *_, **kw):
+            if self.stepped:
+                self.stepped = False
+                se.stop()
+            else:
+                self.stepped = True
+            return True
+
+    class _stackfix:
+        hook: Optional[CodeHook]
+
+        def __init__(self, parent: SpeakeasyEmulator):
+            self.hook = None
+            self.parent = parent
+
+        def __call__(self, base_emu: Se, address: int, size: int, ctx: list):
+            if hook := self.hook:
+                emu = self.parent
+                stack = emu.stack_region
+                emu.sp = stack.base + stack.size // 3
+                hook.disable()
+
     def _reset(self):
         exe = self.exe
         if exe.type not in (ET.PE, ET.BLOB):
@@ -891,10 +951,28 @@ class SpeakeasyEmulator(Emulator[Se, str, _T]):
             else:
                 self.base = emu.load_module(vf.path, data=db).get_base()
 
+        if emu.emu is None:
+            raise RuntimeError('emulator failed to initialize')
+
+        self._end_hook_s = None
+        self._end_hook_d = None
+
+        self._single_step_hook_s = emu.add_code_hook(self._singlestep())
+        self._single_step_hook_d = emu.add_dyn_code_hook(self._singlestep())
+        self._disable_single_step()
+
+        stackfix = self._stackfix(self)
+        stackfix.hook = emu.add_code_hook(stackfix)
+
         emu.emu.timeout = 0
 
+        # prevent memory hook from being overridden, this is a bug in speakeasy
+        emu.emu.add_interrupt_hook(cb=emu.emu._hook_interrupt)
+        emu.emu.builtin_hooks_set = True
+
         if self.hooked(Hook.CodeExecute):
             emu.add_code_hook(self.hook_code_execute, ctx=self.state)
+            emu.add_dyn_code_hook(self.hook_code_execute, ctx=self.state)
 
         if self.hooked(Hook.MemoryRead):
             emu.add_mem_read_hook(self.hook_mem_read)
@@ -908,6 +986,22 @@ class SpeakeasyEmulator(Emulator[Se, str, _T]):
         if self.hooked(Hook.ApiCall):
             emu.add_api_hook(self.hook_api_call, '*', '*')
 
+    def _enable_single_step(self):
+        hd = self._single_step_hook_d
+        hs = self._single_step_hook_s
+        if hd is None or hs is None:
+            raise RuntimeError('single stepping hooks failed to be installed')
+        hd.cb.stepped = False
+        hs.cb.stepped = False
+        hd.enable()
+        hs.enable()
+
+    def _disable_single_step(self):
+        if hook := self._single_step_hook_d:
+            hook.disable()
+        if hook := self._single_step_hook_s:
+            hook.disable()
+
     @property
     def stack_region(self):
         emu = self.speakeasy
@@ -953,42 +1047,87 @@ class SpeakeasyEmulator(Emulator[Se, str, _T]):
         stack.base = base
         stack.size = stack.size + self.alloc_size
 
+    class _stop:
+        hook: Optional[SeHook]
+        address: Optional[int]
+
+        def __init__(self, address: Optional[int] = None):
+            self.address = address
+            self.hook = None
+
+        def __call__(self, spky: Se, address: int, size: int, ctx: list):
+            if hook := self.hook:
+                if address == self.address:
+                    spky.stop()
+                    hook.disable()
+
+    _end_hook_s: Optional[_stop]
+    _end_hook_d: Optional[_stop]
+
+    def _remove_hook(self, hook: Optional[SeHook]):
+        if hook is None:
+            return
+        hook.emu_eng.hook_del(hook.handle)
+        emu = self.speakeasy.emu
+        assert emu is not None
+        for hooklist in emu.hooks.values():
+            assert isinstance(hooklist, list)
+            for k, h in enumerate(hooklist):
+                if h is hook:
+                    del hooklist[k]
+                    break
+
+    def _set_end(self, end: Optional[int]):
+        if h := self._end_hook_s:
+            self._remove_hook(h.hook)
+        if h := self._end_hook_d:
+            self._remove_hook(h.hook)
+        if end is None:
+            self._end_hook_s = None
+            self._end_hook_d = None
+        else:
+            self._end_hook_s = h = self._stop(end)
+            h.hook = self.speakeasy.add_code_hook(h, end, end + 1)
+            self._end_hook_d = h = self._stop(end)
+            h.hook = self.speakeasy.add_dyn_code_hook(h)
+
     def _emulate(self, start: int, end: Optional[int] = None):
-        emu = self.speakeasy
+        spk = self.speakeasy
+        inner = spk.emu
+        assert inner
+        self._set_end(end)
 
-        def stackfix(emu, address: int, size: int, ctx: list):
-            if not ctx:
-                stack = self.stack_region
-                self.sp = stack.base + stack.size // 3
-                ctx.append(True)
-            return True
-
-        emu.add_code_hook(stackfix, start, start, ctx=[])
-
-        if end is not None:
-            def _terminate(*_):
-                emu.stop()
-            emu.add_code_hook(_terminate, end, end + 1)
+        if inner.get_current_run():
+            return spk.resume(start)
 
         if self.exe.blob:
-            emu.run_shellcode(start)
+            offset = start - self.base
+            if offset < 0:
+                raise ValueError(F'invalid offset 0x{start:X} specified; base address is 0x{self.base:X}')
+            spk.run_shellcode(self.base, offset=offset)
         else:
-            inner = emu.emu
+
             inner.stack_base, stack_addr = inner.alloc_stack(self.stack_size)
             inner.set_func_args(inner.stack_base, inner.return_hook)
+
             run = se_profiler.Run()
             run.type = 'thread'
             run.start_addr = start
             run.instr_cnt = 0
             run.args = ()
+
             inner.add_run(run)
+
             if not (p := inner.init_container_process()):
                 p = se_objman.Process(self)
+
             inner.processes.append(p)
             inner.curr_process = p
             if mm := inner.get_address_map(start):
                 mm.set_process(inner.curr_process)
+
             t = se_objman.Thread(inner, stack_base=inner.stack_base, stack_commit=self.stack_size)
+
             inner.om.objects.update({t.address: t})
             inner.curr_process.threads.append(t)
             inner.curr_thread = t
