@@ -18,10 +18,17 @@ from typing import NamedTuple, TYPE_CHECKING
 
 from abc import ABC, abstractmethod
 from enum import Enum
-from uuid import uuid4
+from uuid import uuid4, UUID
+from functools import lru_cache
 
 from refinery.lib import lief
 from refinery.lib.types import INF, ByteStr
+from refinery.lib.exceptions import MissingModule
+
+try:
+    import capstone as cs
+except ImportError:
+    cs = MissingModule('capstone')
 
 if TYPE_CHECKING:
     from lief.ELF import Binary as ELFBinary
@@ -29,6 +36,8 @@ if TYPE_CHECKING:
     from lief.MachO import FatBinary as MachOFatBinary
     from lief.PE import Binary as PEBinary
 
+    from capstone import Cs
+
     from typing import (
         ClassVar,
         Generator,
@@ -101,7 +110,9 @@ class Range(NamedTuple):
     def __len__(self):
         return self.upper - self.lower
 
-    def __contains__(self, addr: int):
+    def __contains__(self, addr: object):
+        if not isinstance(addr, int):
+            raise TypeError
         return self.lower <= addr < self.upper
 
     def __str__(self):
@@ -153,7 +164,7 @@ class ArchItem(NamedTuple):
     An item of the `refinery.lib.executable.Arch` enumeration. It is used to store the register
     size in bits for a given architecture.
     """
-    id: int
+    id: UUID
     pointer_size: int
 
     @classmethod
@@ -427,6 +438,18 @@ class Executable(ABC):
         """
         return self.lookup_location(offset, LT.PHYSICAL)
 
+    def offset_to_address(self, offset: int):
+        """
+        Return the virtual address corresponding to a file offset.
+        """
+        return self.location_from_offset(offset).virtual.position
+
+    def address_to_offset(self, address: int):
+        """
+        Return the virtual address corresponding to a file offset.
+        """
+        return self.location_from_address(address).physical.position
+
     def image_defined_size(self) -> int:
         """
         Returns the size of the executable on disk.
@@ -550,6 +573,41 @@ class Executable(ABC):
                 name = F'synthesized/.zeros-{gap.lower:08X}'
             yield Section(name, gap, Range(v_lower, v_upper), True)
 
+    def disassemble_instruction(self, address: int):
+        """
+        Disassemble a single instruction at the given address.
+        """
+        cs = self.disassembler()
+        cs.detail = True
+        data = self[address:address + 0x20]
+        return next(cs.disasm(data, address, 1))
+
+    @lru_cache
+    def disassembler(self) -> Cs:
+        """
+        Create a capstone disassembler that matches the emulator's architecture.
+        """
+        cs_arch, cs_mode = {
+            Arch.X32     : (cs.CS_ARCH_X86,   cs.CS_MODE_32),     # noqa
+            Arch.X64     : (cs.CS_ARCH_X86,   cs.CS_MODE_64),     # noqa
+            Arch.ARM32   : (cs.CS_ARCH_ARM,   cs.CS_MODE_ARM),    # noqa
+            Arch.ARM64   : (cs.CS_ARCH_ARM,   cs.CS_MODE_THUMB),  # noqa
+            Arch.MIPS16  : (cs.CS_ARCH_MIPS,  cs.CS_MODE_16),     # noqa
+            Arch.MIPS32  : (cs.CS_ARCH_MIPS,  cs.CS_MODE_32),     # noqa
+            Arch.MIPS64  : (cs.CS_ARCH_MIPS,  cs.CS_MODE_64),     # noqa
+            Arch.PPC32   : (cs.CS_ARCH_PPC,   cs.CS_MODE_32),     # noqa
+            Arch.PPC64   : (cs.CS_ARCH_PPC,   cs.CS_MODE_64),     # noqa
+            Arch.SPARC32 : (cs.CS_ARCH_SPARC, cs.CS_MODE_32),     # noqa
+            Arch.SPARC64 : (cs.CS_ARCH_SPARC, cs.CS_MODE_V9),     # noqa
+        }[self.arch()]
+
+        cs_mode |= {
+            BO.BE: cs.CS_MODE_BIG_ENDIAN,
+            BO.LE: cs.CS_MODE_LITTLE_ENDIAN,
+        }[self.byte_order()]
+
+        return cs.Cs(cs_arch, cs_mode)
+
 
 class ExecutableCodeBlob(Executable):
     """
@@ -678,12 +736,9 @@ class LIEF(Executable):
         if self._type == ET.PE:
             v_lower += self.image_defined_base()
         v_lower = self.rebase_img_to_usr(v_lower)
-        try:
-            alignment = section.alignment
-        except AttributeError:
-            v_upper = v_lower + section.size
-        else:
-            v_upper = v_lower + align(alignment, section.size)
+        v_align = getattr(section, 'alignment', 1)
+        v__size = getattr(section, 'virtual_size', section.size)
+        v_upper = v_lower + align(v_align, v__size)
         name = self.ascii(section.name)
         if segment_name is not None:
             name = F'{segment_name}/{name}'
