@@ -1,21 +1,23 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-from refinery.units.formats.pe.dotnet import Arg, JSONEncoderUnit
+from refinery.units import Arg
+from refinery.units.formats.pe.dotnet import DotNetJSONEncoderUnit
 from refinery.lib.dotnet.deserialize import BinaryFormatterParser
 
 
-class dnds(JSONEncoderUnit):
+class dnds(DotNetJSONEncoderUnit):
     """
     Stands for "DotNet DeSerialize": Expects data that has been serialized using the .NET class
     "BinaryFormatter". The output is a representation of the deserialized data in JSON format.
     """
 
     def __init__(
-        self, dereference: Arg.Switch('-r', '--keep-references', off=True,
+        self,
+        dereference: Arg.Switch('-r', '--keep-references', off=True,
             help='Do not resolve Object references in serialized data.') = True,
-        encode=None, digest=None
+        encode=None, digest=None, arrays=False
     ):
-        super().__init__(encode=encode, digest=digest, dereference=dereference)
+        super().__init__(encode=encode, digest=digest, arrays=arrays, dereference=dereference)
 
     def process(self, data):
         self.log_debug('initializing parser, will fail on malformed stream')
